//
//  DriverLicense.m
//  CodeLib
//
//  Created by Александр Ушаков on 15/05/14.
//  Copyright (c) 2014 Александр Ушаков. All rights reserved.
//

#include <mach/mach_host.h>
#import "ScannerViewController.h"
#import <BarcodeScanner/Barcode2DScanner.h>

@implementation ScannerViewController
{
    NSString * lastFormat;
    AVCaptureSession *_captureSession;
    AVCaptureDevice *_videoDevice;
    AVCaptureDeviceInput *_videoInput;
    AVCaptureVideoPreviewLayer *_previewLayer;
    BOOL _running;
    AVCaptureMetadataOutput *_metadataOutput;
    NSTimer *focusTimer;
    BOOL detected;
    NSString *decodeResult;
    Barcode2DScanner* scanner;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setupCaptureSession];
    //    _previewLayer.frame = _previewView.bounds;
    _previewLayer.frame = self.view.layer.bounds;
    [self.view.layer addSublayer:_previewLayer];
    
    scanner = [[Barcode2DScanner alloc] init];
    [scanner registerCode: @"cDoLmB.HtIaPuHrDuEnNaA.EpKdNfG4L1K7E"];
    
    // listen for going into the background and stop the session
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillEnterForeground:) name:UIApplicationWillEnterForegroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationDidEnterBackground:) name:UIApplicationDidEnterBackgroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver: self selector:@selector(decodeResultNotification:) name: DecoderResultNotification object: nil];
    [self CustomeOverlay];
    focusTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(reFocus) userInfo:nil repeats:YES];
    // set default allowed barcode types, remove types via setings menu if you don't want them to be able to be scanned
    self.allowedBarcodeTypes = [NSMutableArray new];
    [self.allowedBarcodeTypes addObject:@"org.iso.QRCode"];
    [self.allowedBarcodeTypes addObject:@"org.iso.PDF417"];
    [self.allowedBarcodeTypes addObject:@"org.gs1.UPC-E"];
    [self.allowedBarcodeTypes addObject:@"org.iso.Aztec"];
    [self.allowedBarcodeTypes addObject:@"org.iso.Code39"];
    [self.allowedBarcodeTypes addObject:@"org.iso.Code39Mod43"];
    [self.allowedBarcodeTypes addObject:@"org.gs1.EAN-13"];
    [self.allowedBarcodeTypes addObject:@"org.gs1.EAN-8"];
    [self.allowedBarcodeTypes addObject:@"com.intermec.Code93"];
    [self.allowedBarcodeTypes addObject:@"org.iso.Code128"];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [self startRunning];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [self stopRunning];
    if (focusTimer){
        [focusTimer invalidate];
        focusTimer = nil;
    }
}

- (void) CustomeOverlay
{
    [self.view bringSubviewToFront:self.btn];
    [self.view bringSubviewToFront:self.demoLbl];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}
#pragma mark - AV capture methods

- (void)setupCaptureSession {
    if (_captureSession) return;
    _videoDevice = [AVCaptureDevice
                    defaultDeviceWithMediaType:AVMediaTypeVideo];
    if (!_videoDevice) {
        NSLog(@"No video camera on this device!");
        return;
    }
    _captureSession = [[AVCaptureSession alloc] init];
    _videoInput = [[AVCaptureDeviceInput alloc]
                   initWithDevice:_videoDevice error:nil];
    if ([_captureSession canAddInput:_videoInput]) {
        [_captureSession addInput:_videoInput];
    }
    _previewLayer = [[AVCaptureVideoPreviewLayer alloc]
                     initWithSession:_captureSession];
    _previewLayer.videoGravity =
    AVLayerVideoGravityResizeAspectFill;
    
    _metadataOutput = [[AVCaptureMetadataOutput alloc] init];
    //    dispatch_queue_t metadataQueue =
    //    dispatch_queue_create("com.1337labz.featurebuild.metadata", 0);
    [_metadataOutput setMetadataObjectsDelegate:self
                                          queue:dispatch_get_main_queue()];
    if ([_captureSession canAddOutput:_metadataOutput]) {
        [_captureSession addOutput:_metadataOutput];
    }
}

- (void)startRunning {
    if (_running) return;
    [_captureSession startRunning];
    _metadataOutput.metadataObjectTypes =
    _metadataOutput.availableMetadataObjectTypes;
    _running = YES;
}
- (void)stopRunning {
    if (!_running) return;
    [_captureSession stopRunning];
    _running = NO;
}

//  handle going foreground/background
- (void)applicationWillEnterForeground:(NSNotification*)note {
    [self startRunning];
}
- (void)applicationDidEnterBackground:(NSNotification*)note {
    [self stopRunning];
}

- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputMetadataObjects:(NSArray *)metadataObjects fromConnection:(AVCaptureConnection *)connection
{
    if (detected) {
        return;
    }
    
    for (AVMetadataObject *metadata in metadataObjects) {
        if ([metadata isKindOfClass:
             [AVMetadataMachineReadableCodeObject class]])
        {
            AVMetadataMachineReadableCodeObject *code = (AVMetadataMachineReadableCodeObject*) [_previewLayer transformedMetadataObjectForMetadataObject:metadata];
            
            NSString* result = [scanner scanGrayscaleImage: code Width: [[UIScreen mainScreen] bounds].size.width Height: [[UIScreen mainScreen] bounds].size.height Encoding:NSUTF8StringEncoding ];
            
            NSLog(@"Frame decoded");
            
            //ignore results less than 4 characters - probably false detection
            if ( [result length] > 4 )
            {
                NSLog(@"Detected: %@", result);
                detected = YES;
                
                [self stopRunning];
                NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
                DecoderResult *notificationResult = [DecoderResult createSuccess:result];
                [center postNotificationName:DecoderResultNotification object: notificationResult];
                detected = NO;
            }
            else
            {
                detected = NO;
            }
        }
    }
}

// IOS 7 statusbar hide
- (BOOL)prefersStatusBarHidden
{
    return YES;
}

-(void) reFocus {
    //NSLog(@"refocus");
    
    NSError *error;
    if ([_videoDevice lockForConfiguration:&error]) {
        
        if ([_videoDevice isFocusPointOfInterestSupported]){
            [_videoDevice setFocusPointOfInterest:CGPointMake(0.49,0.49)];
            [_videoDevice setFocusMode:AVCaptureFocusModeAutoFocus];
        }
        [_videoDevice unlockForConfiguration];
        
    }
}

- (void)decodeResultNotification: (NSNotification *)notification {
    
    if ([notification.object isKindOfClass:[DecoderResult class]])
    {
        DecoderResult *obj = (DecoderResult*)notification.object;
        if (obj.succeeded)
        {
            decodeResult = [[NSString alloc] initWithString:obj.result];
            UIAlertView * messageDlg = [[UIAlertView alloc] initWithTitle:[NSString stringWithFormat:@"Format: %@",lastFormat] message:decodeResult
                                                                 delegate:self cancelButtonTitle:nil otherButtonTitles:@"Close", nil];
            [messageDlg show];
        }
    }
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {
    if (buttonIndex == 0) {
        [self startRunning];
    }
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    UIInterfaceOrientation interfaceOrientation =[[UIApplication sharedApplication] statusBarOrientation];
    
    switch (interfaceOrientation) {
        case UIInterfaceOrientationPortrait:
            return UIInterfaceOrientationMaskPortrait;
            break;
        case UIInterfaceOrientationPortraitUpsideDown:
            return UIInterfaceOrientationMaskPortraitUpsideDown;
            break;
        case UIInterfaceOrientationLandscapeLeft:
            return UIInterfaceOrientationMaskLandscapeLeft;
            break;
        case UIInterfaceOrientationLandscapeRight:
            return UIInterfaceOrientationMaskLandscapeRight;
            break;
            
        default:
            break;
    }
    
    return UIInterfaceOrientationMaskAll;
    
}

- (BOOL) shouldAutorotate {
    
    return YES;
    
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)toggleTorch
{
    if ([_videoDevice isTorchModeSupported:AVCaptureTorchModeOn]) {
        NSError *error;
        
        if ([_videoDevice lockForConfiguration:&error]) {
            if ([_videoDevice torchMode] == AVCaptureTorchModeOn)
                [_videoDevice setTorchMode:AVCaptureTorchModeOff];
            else
                [_videoDevice setTorchMode:AVCaptureTorchModeOn];
            
            if([_videoDevice isFocusModeSupported: AVCaptureFocusModeContinuousAutoFocus])
                _videoDevice.focusMode = AVCaptureFocusModeContinuousAutoFocus;
            
            [_videoDevice unlockForConfiguration];
        } else {
            
        }
    }
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    [self toggleTorch];
    
}

- (IBAction)closeBtn:(id)sender {
    [self stopRunning];
    [self dismissViewControllerAnimated:YES completion:nil];
}
@end

/*
 *  Implementation of the object that returns decoder results (via the notification
 *	process)
 */

@implementation DecoderResult

@synthesize succeeded;
@synthesize result;

+(DecoderResult *)createSuccess:(NSString *)result {
    DecoderResult *obj = [[DecoderResult alloc] init];
    if (obj != nil) {
        obj.succeeded = YES;
        obj.result = result;
    }
    return obj;
}

+(DecoderResult *)createFailure {
    DecoderResult *obj = [[DecoderResult alloc] init];
    if (obj != nil) {
        obj.succeeded = NO;
        obj.result = nil;
    }
    return obj;
}

- (void)dealloc {
#if !__has_feature(objc_arc)
    [super dealloc];
#endif
    self.result = nil;
}

@end
